@@ -23,6 +23,8 @@
 # -- changed rgl.* to *3d functions
 # last revised: 11/5/2008 by MF
 # -- added sufix= to heplot.candisc and heplot3d.candisc
+# last revised: 11/12/2008 by MF
+# -- added asp= to heplot3d.candisc
 
 ## TODO:
 
@@ -96,16 +98,16 @@ heplot.candisc <- function (
 }
 
 ## heplot3d method for candisc object
-## TODO:
-#    - How to set par3d(scale) or aspect3d based on bbox of E matrix?
+## TODO: How to set par3d(scale) or aspect3d based on bbox of E matrix?
 #      (This should be an option, because sometimes the equal-scaling
 #       dimensions will be extremely thin on the 3rd dimension.)
-#    - Complete the calculation of scale when missing
+#  TODO: Complete the calculation of scale when missing
 
 heplot3d.candisc <- function (
 	mod,		    # output object from candisc
 	which=1:3,  # canonical dimensions to plot
 	scale,       # scale factor for variable vectors in can space
+	asp="iso",           # aspect ratio, to ensure equal units
 	var.col="blue",
 	var.lwd=par("lwd"),
 	prefix = "Can",  # prefix for labels of canonical dimensions
@@ -134,7 +136,7 @@ heplot3d.candisc <- function (
   	terms <- lm.terms
 	}
 
-  heplot3d(can.mod, terms=terms,
+  ellipses <-heplot3d(can.mod, terms=terms,
   		factor.means=term,
   		xlab=canlab[1], ylab=canlab[2], zlab=canlab[3], ...)
 
@@ -142,8 +144,11 @@ heplot3d.candisc <- function (
 	maxrms <- function(x) { max(sqrt(apply(x^2, 1, sum))) }
 	if (missing(scale)) {
 		vecmax <- maxrms(structure)
-    # get bbox of the 3d plot
-    bbox <- matrix(par3d("bbox"),3,2,byrow=TRUE)
+		vecrange <- range(structure)
+		ellrange <- lapply(ellipses, range)
+		vecmax <- maxrms(structure)
+		# get bbox of the 3d plot
+        bbox <- matrix(par3d("bbox"),3,2,byrow=TRUE)
 #    TODO: calculate scale so that vectors reasonably fill the plot
 		scale <- 5
 		cat("Vector scale factor set to ", scale, "\n")
@@ -162,6 +167,7 @@ heplot3d.candisc <- function (
 #  rgl.texts( cs, text=rownames(cs), col=var.col)
   texts3d( cs, text=rownames(cs), col=var.col)
 
+  if (!is.null(asp)) aspect3d(asp)
 }
 
 heplot.candiscList <- function(mod, term, ask=interactive(), graphics = TRUE, ...) {
